@@ -239,7 +239,7 @@ IOCompactDiscServices::message ( UInt32 		type,
 		
 		case kSCSIServicesNotification_ExclusivityChanged:
 		case kIOMessageMediaStateHasChanged:
-		case kIOMessageTrayStateHasChanged:
+		case kIOMessageTrayStateChange:
 		case kIOMessageMediaAccessChange:
 		{
 			
@@ -289,9 +289,12 @@ IOCompactDiscServices::setProperties ( OSObject * properties )
 	
 	// The user client is active, reject this call.
 	userClientActive = fProvider->GetUserClientExclusivityState ( );
-	require_action ( ( userClientActive == false ),
-					 ReleaseProvider,
-					 status = kIOReturnExclusiveAccess );
+	require_action (
+		( userClientActive == false ),
+		ReleaseProvider,
+		status = kIOReturnExclusiveAccess;
+		messageClients ( kIOMessageTrayStateChange,
+						 ( void * ) kMessageTrayStateChangeRequestRejected ) );
 	
 	fProvider->CheckPowerState ( );
 	
@@ -343,12 +346,13 @@ IOCompactDiscServices::doAsyncReadCD ( 	IOMemoryDescriptor *	buffer,
 		IOSimpleLockLock ( fDataCacheLock );
 		if ( fDataCacheBlockCount != 0 )
 		{
-			// Check to see if this request could possibly fulfilled by the data
+			
+			// Check to see if this request could possibly be fulfilled by the data
 			// that is currently in the cache.
-			// This is possible if the following conditions appply:
+			// This is possible if the following conditions apply:
 			// 1. The request is the same or smaller than the number of blocks that currently
 			// resides in the cache.
-			// 2. The starting request block is greater than the startiing block of the cache.
+			// 2. The starting request block is greater than the starting block of the cache.
 			// 3. The ending request block is the same or less than the end block in the cache.
 			if ( ( nblks <= fDataCacheBlockCount ) && ( block >= fDataCacheStartBlock ) && 
 				( block + nblks <= fDataCacheStartBlock + fDataCacheBlockCount ) )
@@ -372,14 +376,14 @@ IOCompactDiscServices::doAsyncReadCD ( 	IOMemoryDescriptor *	buffer,
 			}
 			
 		}
-
+		
 		// Release the lock for the Queue
 		IOSimpleLockUnlock ( fDataCacheLock );
 		
 	}
 #endif
 	
-	clientData = ( BlockServicesClientData * ) IOMalloc ( sizeof ( BlockServicesClientData ) );
+	clientData = IONew ( BlockServicesClientData, 1 );
 	require_nonzero_action ( clientData, ErrorExit, status = kIOReturnNoResources );
 	
 	// Make sure we don't go away while the command in being executed.
@@ -492,8 +496,8 @@ IOCompactDiscServices::doAsyncReadCD ( 	IOMemoryDescriptor *	buffer,
 	// avoid doing any cache operations in the completion.
 	clientData->transferSegBuffer 	= NULL;
 	clientData->transferSegDesc 	= NULL;
-	clientData->transferStart 		= 0;
-	clientData->transferCount		= 0;
+	clientData->transferStart 		= block;
+	clientData->transferCount		= nblks;
 #endif
 	
 	status = fProvider->AsyncReadCD ( buffer,
@@ -529,7 +533,7 @@ IOCompactDiscServices::doAsyncReadWrite ( IOMemoryDescriptor *		buffer,
 	
 	require ( ( isInactive ( ) == false ), ErrorExit );
 	
-	clientData = ( BlockServicesClientData * ) IOMalloc ( sizeof ( BlockServicesClientData ) );
+	clientData = IONew ( BlockServicesClientData, 1 );
 	require_nonzero_action ( clientData, ErrorExit, status = kIOReturnNoResources );
 	
 	// Make sure we don't go away while the command in being executed.
@@ -556,10 +560,10 @@ IOCompactDiscServices::doAsyncReadWrite ( IOMemoryDescriptor *		buffer,
 #if (_USE_DATA_CACHING_)
 	// Make sure that this is cleared out to
 	// avoid doing any cache operations in the completion.
-	clientData->transferSegBuffer = NULL;
-	clientData->transferSegDesc = NULL;
-	clientData->transferStart = 0;
-	clientData->transferCount = 0;
+	clientData->transferSegBuffer	= NULL;
+	clientData->transferSegDesc 	= NULL;
+	clientData->transferStart		= 0;
+	clientData->transferCount		= 0;
 #endif
 	
 	status = fProvider->AsyncReadWrite ( buffer, block, nblks, ( void * ) clientData );
@@ -1613,24 +1617,48 @@ IOCompactDiscServices::AsyncReadWriteComplete ( void * 			clientData,
 		bsClientData->retriesLeft--;
 		if ( bsClientData->clientReadCDCall == true )
 		{
-		
+			
 #if (_USE_DATA_CACHING_)
-			requestStatus = owner->fProvider->AsyncReadCD ( 
+			
+			if ( ( bsClientData->transferSegDesc != NULL ) &&
+				 ( bsClientData->transferCount != 0 ) )
+			{
+				
+				requestStatus = owner->fProvider->AsyncReadCD (
 									bsClientData->transferSegDesc,
 									bsClientData->transferStart,
 									bsClientData->transferCount,
 									bsClientData->clientSectorArea,
 									bsClientData->clientSectorType,
 									clientData );
-#else
-			requestStatus = owner->fProvider->AsyncReadCD (
-									bsClientData->clientBuffer, 
-									bsClientData->clientStartingBlock, 
-									bsClientData->clientRequestedBlockCount, 
+				
+			}
+			
+			else
+			{
+				
+				requestStatus = owner->fProvider->AsyncReadCD (
+									bsClientData->clientBuffer,
+									bsClientData->clientStartingBlock,
+									bsClientData->clientRequestedBlockCount,
 									bsClientData->clientSectorArea,
 									bsClientData->clientSectorType,
 									clientData );
-#endif
+				
+			}
+
+#else	/* !_USE_DATA_CACHING_ */
+			
+			requestStatus = owner->fProvider->AsyncReadCD (
+									bsClientData->clientBuffer,
+									bsClientData->clientStartingBlock,
+									bsClientData->clientRequestedBlockCount,
+									bsClientData->clientSectorArea,
+									bsClientData->clientSectorType,
+									clientData );
+			
+#endif	/* _USE_DATA_CACHING_ */
+		
 		}
 		
 		else
@@ -1724,7 +1752,7 @@ IOCompactDiscServices::AsyncReadWriteComplete ( void * 			clientData,
 		}
 #endif
 		
-		IOFree ( clientData, sizeof ( BlockServicesClientData ) );
+		IODelete ( clientData, BlockServicesClientData, 1 );
 		
 		// Release the retain for this command.	
 		owner->fProvider->release ( );
