@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998-2001 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2002 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -20,10 +20,32 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 
+
+//
+//	Includes
+//
+
+// SCSI Architecture Model Family includes
 #include <IOKit/scsi-commands/SCSITask.h>
+#include <IOKit/IOMessage.h>
+#include <IOKit/pwr_mgt/RootDomain.h>
 #include "IOSCSIMultimediaCommandsDevice.h"
 
-#define SCSI_MMC_DEVICE_DEBUGGING_LEVEL 0
+
+//
+//	Macros
+//
+
+#define DEBUG 									0
+#define DEBUG_ASSERT_COMPONENT_NAME_STRING		"MMCPower"
+
+#if DEBUG
+#define SCSI_MMC_DEVICE_DEBUGGING_LEVEL			0
+#endif
+
+
+#include "IOSCSIArchitectureModelFamilyDebugging.h"
+
 
 #if ( SCSI_MMC_DEVICE_DEBUGGING_LEVEL >= 1 )
 #define PANIC_NOW(x)		IOPanic x
@@ -44,14 +66,19 @@
 #endif
 
 #define super IOSCSIPrimaryCommandsDevice
-#define kPowerStatusBufferSize	8
-#define kPowerStatusByte		5
 
-//---------------------------------------------------------------------------
+
+//
+//	Constants
+//
+
+#define kPowerStatusBufferSize		8
+#define kPowerStatusByte			5
+
+
 // We have 4 default power states. ACTIVE, IDLE, STANDBY, and SLEEP in order
 // of most power to least power. In SLEEP, the Vcc can be removed, so we must
 // trap for that in our power management code.
-//---------------------------------------------------------------------------
 
 static IOPMPowerState sPowerStates[kMMCNumPowerStates] =
 {
@@ -62,12 +89,146 @@ static IOPMPowerState sPowerStates[kMMCNumPowerStates] =
 	{ kIOPMPowerStateVersion1, (IOPMDeviceUsable | IOPMMaxPerformance | kIOPMPreventIdleSleep), IOPMPowerOn, IOPMPowerOn, 0, 0, 0, 0, 0, 0, 0, 0 }
 };
 
+enum
+{
+	kMMCPowerConditionsActive	= 0x01,
+	kMMCPowerConditionsIdle		= 0x02,
+	kMMCPowerConditionsStandby	= 0x03,
+	kMMCPowerConditionsSleep 	= 0x05
+};
 
-//---------------------------------------------------------------------------
+
+// Static prototypes
+static IOReturn
+IOSCSIMultimediaCommandsDevicePowerDownHandler ( void * 		target,
+												 void * 		refCon,
+												 UInt32 		messageType,
+												 IOService * 	provider,
+												 void * 		messageArgument,
+												 vm_size_t 		argSize );
+
+
+//
+//  IOSCSIMultimediaCommandsDevicePowerDownHandler -	C->C++ Glue code for
+// 													 	Power Down
+//														Notifications. [STATIC]
+//
+
+
+static IOReturn
+IOSCSIMultimediaCommandsDevicePowerDownHandler ( void * 		target,
+												 void * 		refCon,
+												 UInt32 		messageType,
+												 IOService * 	provider,
+												 void * 		messageArgument,
+												 vm_size_t 		argSize )
+{
+	
+	return ( ( IOSCSIMultimediaCommandsDevice * ) target )->PowerDownHandler (
+														refCon,
+														messageType,
+														provider,
+														messageArgument,
+														argSize );
+	
+}
+
+
+//
+//  PowerDownHandler - Method called at sleep/restart/shutdown time.
+//																	[PROTECTED]
+//
+
+IOReturn
+IOSCSIMultimediaCommandsDevice::PowerDownHandler ( void * 		refCon,
+												   UInt32 		messageType,
+												   IOService * 	provider,
+												   void * 		messageArgument,
+												   vm_size_t 	argSize )
+{
+	
+	IOReturn				status 		= kIOReturnUnsupported;
+	SCSITaskIdentifier		request		= NULL;
+	
+	// Make sure it is a power off message first.
+	// We don't do anything at sleep or restart time. That is
+	// handled via standard power management functions. See
+	// HandlePowerChange() for more information.
+	
+	require ( messageType == kIOMessageSystemWillPowerOff, ErrorExit );
+	
+	request = GetSCSITask ( );
+	require_nonzero ( request, ErrorExit );
+	
+	// This is our chance to bring the drive tray in (if the drive supports
+	// power inject).
+	if ( fMediaPresent == false )
+	{
+		
+		UInt8	trayState = 0;	// 0 = close, 1 = open
+		
+		// Bring the tray closed if the power state is not in some sleep mode.			
+		if ( fCurrentPowerState > kMMCPowerStateSleep )
+		{
+			
+			status = GetTrayState ( &trayState );
+			if ( ( ( status == kIOReturnSuccess ) && ( trayState == 1 ) ) ||
+				 ( status != kIOReturnSuccess ) )
+			{
+				
+				// Set the tray state to closed						
+				if ( START_STOP_UNIT ( request, 0, 0, 1, 1, 0 ) == true )
+				{
+					( void ) SendCommand ( request, kTenSecondTimeoutInMS );
+				}
+				
+				status = kIOReturnSuccess;
+				
+			}
+			
+		}
+		
+	}
+	
+	else
+	{
+		
+		// Media is present (but may be spinning). Make sure
+		// the drive is spun down.
+		if ( fCurrentPowerState > kMMCPowerStateSleep )
+		{
+			
+			// Make sure the drive is spun down
+			if ( START_STOP_UNIT ( request, 1, 0, 0, 0, 0 ) == true )
+			{
+				
+				( void ) SendCommand ( request, 0 );
+				
+			}
+			
+		}
+		
+		status = kIOReturnSuccess;
+		
+	}
+	
+	ReleaseSCSITask ( request );
+	
+	
+ErrorExit:
+	
+	
+	return status;
+	
+}
+
+
+//
 //  GetInitialPowerState - Asks the driver which power state the device is
 //							in at startup time. This function is only called
 //							once, right after InitializePowerManagement().
-//---------------------------------------------------------------------------
+//																	[PROTECTED]
+//
 
 UInt32
 IOSCSIMultimediaCommandsDevice::GetInitialPowerState ( void )
@@ -78,11 +239,11 @@ IOSCSIMultimediaCommandsDevice::GetInitialPowerState ( void )
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  GetNumberOfPowerStateTransitions - Asks the driver for the number of state
-//					transitions between sleep state and
-//					the highest power state.
-//---------------------------------------------------------------------------
+//										transitions between sleep state and
+//										the highest power state.	[PROTECTED]
+//
 
 UInt32
 IOSCSIMultimediaCommandsDevice::GetNumberOfPowerStateTransitions ( void )
@@ -95,10 +256,7 @@ IOSCSIMultimediaCommandsDevice::GetNumberOfPowerStateTransitions ( void )
 	// power states, so we just return 0 power state changes. Otherwise,
 	// we return the normal number of power transitions from active state
 	// to sleep state.
-	
-	
 	state = HandleGetUserClientExclusivityState ( );
-	
 	if ( state == false )
 	{
 		numTransitions = kMMCPowerStateActive - kMMCPowerStateSleep;
@@ -109,17 +267,16 @@ IOSCSIMultimediaCommandsDevice::GetNumberOfPowerStateTransitions ( void )
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  InitializePowerManagement - 	Register the driver with our policy-maker
-//									(also in the same class).
-//---------------------------------------------------------------------------
+//									(also in the same class).		[PROTECTED]
+//
 
 void
-IOSCSIMultimediaCommandsDevice::InitializePowerManagement ( IOService * provider )
+IOSCSIMultimediaCommandsDevice::InitializePowerManagement (
+										IOService * provider )
 {
 	
-	STATUS_LOG ( ( "IOSCSIMultimediaCommandsDevice::InitializePowerManagement called\n" ) );	
-	
 	fCurrentPowerState = kMMCPowerStateActive;
 	
 	// Call our super to get us into the power management tree
@@ -128,7 +285,12 @@ IOSCSIMultimediaCommandsDevice::InitializePowerManagement ( IOService * provider
 	// Register ourselves as a "policy maker" for this device. We use
 	// the number of default power states defined by MMC-2.
 	registerPowerDriver ( this, sPowerStates, kMMCNumPowerStates );
-
+	
+	// Install handler for shutdown notifications
+	fPowerDownNotifier = registerPrioritySleepWakeInterest (
+		( IOServiceInterestHandler ) IOSCSIMultimediaCommandsDevicePowerDownHandler,
+		this );
+	
 	// Make sure we clamp the lowest power setting that we voluntarily go
 	// into is state kMMCPowerStateSleep. We only enter kMMCPowerStateSystemSleep
 	// if told by the power manager during a system sleep.
@@ -137,9 +299,10 @@ IOSCSIMultimediaCommandsDevice::InitializePowerManagement ( IOService * provider
 }
 
 
-//---------------------------------------------------------------------------
-//  HandleCheckPowerState - 
-//---------------------------------------------------------------------------
+//
+//  HandleCheckPowerState - 	Checks to see if the power state is
+//								kMMCPowerStateActive				[PROTECTED]
+//
 
 void
 IOSCSIMultimediaCommandsDevice::HandleCheckPowerState ( void )
@@ -155,10 +318,11 @@ IOSCSIMultimediaCommandsDevice::HandleCheckPowerState ( void )
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  TicklePowerManager - 	Calls activityTickle to tell the power manager we
 //							need to be in a certain state to fulfill I/O
-//---------------------------------------------------------------------------
+//																	[PROTECTED]
+//
 
 void
 IOSCSIMultimediaCommandsDevice::TicklePowerManager ( void )
@@ -176,9 +340,10 @@ IOSCSIMultimediaCommandsDevice::TicklePowerManager ( void )
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  HandlePowerChange - Handles the state machine for power management
-//---------------------------------------------------------------------------
+//																	[PROTECTED]
+//
 
 void
 IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
@@ -199,36 +364,37 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 	 					fProposedPowerState, fCurrentPowerState ) );
 	 	
 		// are we currently asleep and going to wake up?
-	 	if ( ( fCurrentPowerState <= kMMCPowerStateSleep ) && ( fProposedPowerState > kMMCPowerStateSleep ) )
+	 	if ( ( fCurrentPowerState <= kMMCPowerStateSleep ) &&
+	 		 ( fProposedPowerState > kMMCPowerStateSleep ) )
 		{
 			
 			STATUS_LOG ( ( "We think we're in sleep\n" ) );
 			
-			// Test to see if the drive is asleep. Send a TEST_UNIT_READY with a timeout of 1 second.
-			// If it is asleep, it will return a
-			// serviceResponse == kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE. The device
-			// will be reset and that will wake it up.
+			// Test to see if the drive is asleep. Send a TEST_UNIT_READY with
+			// a timeout of 1 second. If it is asleep, it will be reset and
+			// that will wake it up.
 			if ( TEST_UNIT_READY ( request, 0 ) == true )
 			{
-				serviceResponse = SendCommand ( request, 1000 );
+				( void ) SendCommand ( request, kOneSecondTimeoutInMS );
 			}
 			
 			// First, if we were in sleep mode ( fCurrentPowerState <= kMMCPowerStateSleep )
-			// then we need to do some stuff to recover the device. If power was pulled, we
-			// need to clear the power-on reset sense data.
+			// then we need to do some stuff to recover the device. If power
+			// was pulled, we need to clear the power-on reset sense data.
 			STATUS_LOG ( ( "calling ClearPowerOnReset\n" ) );
 			
 			// /!\ WARNING! we may bail out of ClearPowerOnReset because
-			// the device was hot unplugged while we were sleeping - do sanity check here
-			// and break out of loop if we have been terminated.
+			// the device was hot unplugged while we were sleeping - do
+			// sanity check here and break out of loop if we have been
+			// terminated.
 			if ( ClearPowerOnReset ( ) == false )
 				break;
 			
 			STATUS_LOG ( ( "calling ClearNotReadyStatus\n" ) );
 			
 			// /!\ WARNING! we may bail out of ClearNotReadyStatus because
-			// the device was hot unplugged while we were sleeping - do sanity check here
-			// and break out of loop if we have been terminated.
+			// the device was hot unplugged while we were sleeping - do sanity
+			// check here and break out of loop if we have been terminated.
 			if ( ClearNotReadyStatus ( ) == false )
 				break;
 			
@@ -242,9 +408,9 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					
 					// Give the driver some lee-way in waking up because we think
 					// media is actually there!
-					for ( UInt32 index = 0; index < 20; index++ )
+					for ( UInt32 index = 0; index < 100; index++ )
 					{
-
+						
 						STATUS_LOG ( ( "Calling CheckMediaPresence\n" ) );
 						
 						mediaPresent = CheckMediaPresence ( );
@@ -262,21 +428,18 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					
 					else
 					{
-					
-						STATUS_LOG ( ( "Calling PREVENT_ALLOW_MEDIUM_REMOVAL\n" ) );
-	
-						// Media is present, so lock it down
-						if ( PREVENT_ALLOW_MEDIUM_REMOVAL ( request, 1, 0 ) == true )
-					    {
-					    	// The command was successfully built, now send it
-					    	serviceResponse = SendCommand ( request, 0 );
 						
-						}
-						else
+						STATUS_LOG ( ( "Calling PREVENT_ALLOW_MEDIUM_REMOVAL\n" ) );
+						
+						// Media is present, so lock it down
+						if ( PREVENT_ALLOW_MEDIUM_REMOVAL ( request, kMediaStateLocked, 0 ) == true )
 						{
-							PANIC_NOW ( ( "IOSCSIMultimediaCommandsDevice::HandlePowerChange malformed command" ) );
+							
+							// The command was successfully built, now send it
+					    	serviceResponse = SendCommand ( request, kTenSecondTimeoutInMS );
+							
 						}
-					
+						
 					}
 					
 				}
@@ -306,11 +469,11 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 				{
 					
 					STATUS_LOG ( ( "We are already asleep\n" ) );
-
+					
 					// is there media in the drive?
 					if ( fMediaPresent == true )
 					{
-
+						
 						STATUS_LOG ( ( "Media is present, bailing out\n" ) );
 						
 						fCurrentPowerState = kMMCPowerStateSystemSleep;
@@ -356,6 +519,12 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					
 				}
 				
+				// Set the tray state to closed.
+				if ( START_STOP_UNIT ( request, 0, 0, 1, 1, 0 ) == true )
+				{
+					( void ) SendCommand ( request, kTenSecondTimeoutInMS );
+				}
+				
 				// If the device supports the power conditions mode page, and we haven't already
 				// put it to sleep using the START_STOP_UNIT command, issue one to the drive.
 				if ( fDeviceSupportsPowerConditions )
@@ -363,7 +532,7 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					
 					STATUS_LOG ( ( "Sending DVD sleep command\n" ) );
 					
-					if ( START_STOP_UNIT ( request, 0, 0x05, 0, 0, 0 ) == true )
+					if ( START_STOP_UNIT ( request, 0, kMMCPowerConditionsSleep, 0, 0, 0 ) == true )
 					{
 						
 						( void ) SendCommand ( request, 0 );
@@ -378,7 +547,9 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					STATUS_LOG ( ( "Sending ATA sleep command\n" ) );
 					
 					features = true;
-					( void ) HandleProtocolServiceFeature ( kSCSIProtocolFeature_ProtocolSpecificSleepCommand, ( void * ) &features );
+					( void ) HandleProtocolServiceFeature (
+								kSCSIProtocolFeature_ProtocolSpecificSleepCommand,
+								( void * ) &features );
 					
 				}
 				
@@ -392,7 +563,7 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					if ( START_STOP_UNIT ( request, 0, 0, 0, 0, 0 ) == true )
 					{
 						
-						serviceResponse = SendCommand ( request, 0 );
+						( void ) SendCommand ( request, 0 );
 						
 					}
 				
@@ -432,7 +603,13 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 				{
 					
 					STATUS_LOG ( ( "Media is NOT present.\n" ) );
-
+					
+					// Set the tray state to closed.
+					if ( START_STOP_UNIT ( request, 0, 0, 1, 1, 0 ) == true )
+					{
+						( void ) SendCommand ( request, 0 );
+					}
+					
 					// does the device support low power polling?
 					if ( fDeviceSupportsLowPowerPolling == false )
 					{
@@ -444,7 +621,7 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 						if ( START_STOP_UNIT ( request, 0, 0, 0, 0, 0 ) == true )
 						{
 							
-							serviceResponse = SendCommand ( request, 0 );
+							( void ) SendCommand ( request, 0 );
 							
 						}
 						
@@ -489,18 +666,22 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 					STATUS_LOG ( ( "Calling protocol layer to sleep drive\n" ) );
 					
 					features = true;
-					( void ) HandleProtocolServiceFeature ( kSCSIProtocolFeature_ProtocolSpecificSleepCommand, ( void * ) &features );
+					( void ) HandleProtocolServiceFeature (
+								kSCSIProtocolFeature_ProtocolSpecificSleepCommand,
+								( void * ) &features );
 					
 				}
 				
 				// is there media in the drive?
 				if ( fMediaPresent == false )
 				{
-
+					
 					STATUS_LOG ( ( "Enabling low power polling.\n" ) );
 					
 					features = fLowPowerPollingEnabled = true;
-					HandleProtocolServiceFeature ( kSCSIProtocolFeature_ProtocolSpecificPolling, ( void * ) &features );
+					( void ) HandleProtocolServiceFeature (
+								kSCSIProtocolFeature_ProtocolSpecificPolling,
+								( void * ) &features );
 					
 				}
 				
@@ -529,10 +710,10 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 						
 						STATUS_LOG ( ( "Sending START_STOP_UNIT to put drive in standby mode\n" ) );
 						
-						if ( START_STOP_UNIT ( request, 0, 0x03, 0, 0, 0 ) == true )
+						if ( START_STOP_UNIT ( request, 0, kMMCPowerConditionsStandby, 0, 0, 0 ) == true )
 						{
 							
-							serviceResponse = SendCommand ( request, 0 );
+							( void ) SendCommand ( request, 0 );
 							
 						}
 						
@@ -565,10 +746,10 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 						
 						STATUS_LOG ( ( "Sending START_STOP_UNIT to drive to put it in idle mode\n" ) );
 						
-						if ( START_STOP_UNIT ( request, 0, 0x02, 0, 0, 0 ) == true )
+						if ( START_STOP_UNIT ( request, 0, kMMCPowerConditionsIdle, 0, 0, 0 ) == true )
 						{
 							
-							serviceResponse = SendCommand ( request, 0 );
+							( void ) SendCommand ( request, 0 );
 							
 						}
 						
@@ -618,6 +799,14 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 		}
 		
 	}
+
+	if ( isInactive ( ) )
+	{
+		
+		fCurrentPowerState = fProposedPowerState;
+		fCommandGate->commandWakeup ( &fCurrentPowerState, false );
+		
+	}
 	
 	ReleaseSCSITask ( request );
 	
@@ -626,19 +815,19 @@ IOSCSIMultimediaCommandsDevice::HandlePowerChange ( void )
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  CheckMediaPresence - 	Checks if media is present. If so, it locks it down
 //							again, else it requests a dialog to be displayed
-//							and tears the stack down
-//---------------------------------------------------------------------------
+//							and tears the stack down				[PROTECTED]
+//
 
 bool
 IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 {
 	
-	SCSI_Sense_Data				senseBuffer;
-	IOMemoryDescriptor *		bufferDesc;
-	SCSITaskIdentifier			request;
+	SCSI_Sense_Data				senseBuffer		= { 0 };
+	IOMemoryDescriptor *		bufferDesc		= NULL;
+	SCSITaskIdentifier			request			= NULL;
 	bool						mediaPresent	= false;
 	bool						driveReady 		= false;
 	SCSIServiceResponse 		serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
@@ -649,30 +838,25 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 													kSenseDefaultSize,
 													kIODirectionIn );
 	
+	require_nonzero ( bufferDesc, ErrorExit );
+	
 	request = GetSCSITask ( );
+	require_nonzero ( request, ReleaseDescriptor );
 	
 	do
 	{
 				
 		if ( TEST_UNIT_READY ( request, 0 ) == true )
 	    {
-	    	
-	    	STATUS_LOG ( ( "sending TUR.\n" ) );
+			
 	    	// The command was successfully built, now send it
-	    	serviceResponse = SendCommand ( request, 0 );
+	    	serviceResponse = SendCommand ( request, kTenSecondTimeoutInMS );
 	    	
 		}
 		
-		else
-		{
-			PANIC_NOW ( ( "IOSCSIMultimediaCommandsDevice::CheckMediaPresence malformed command" ) );
-		}
-
 		if ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE )
 		{
-			
-			STATUS_LOG ( ( "TUR completed.\n" ) );
-			
+						
 			bool validSense = false;
 			
 			if ( GetTaskStatus ( request ) == kSCSITaskStatus_CHECK_CONDITION )
@@ -685,21 +869,14 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 					if ( REQUEST_SENSE ( request, bufferDesc, kSenseDefaultSize, 0  ) == true )
 				    {
 				    	
-				    	STATUS_LOG ( ( "sending REQ_SENSE.\n" ) );
 				    	// The command was successfully built, now send it
-				    	serviceResponse = SendCommand ( request, 0 );
+				    	serviceResponse = SendCommand ( request, kTenSecondTimeoutInMS );
 				    	
 					}
 					
-					else
-					{
-						PANIC_NOW ( ( "IOSCSIMultimediaCommandsDevice::CheckMediaPresence malformed command" ) );
-					}
-
 					if ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE )
 					{
 						
-						STATUS_LOG ( ( "validSense data.\n" ) );
 						validSense = true;
 						
 					}
@@ -710,7 +887,7 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 				{
 					
 					STATUS_LOG ( ( "sense data: %01x, %02x, %02x\n",
-								( senseBuffer.SENSE_KEY  & kSENSE_KEY_Mask ),
+								( senseBuffer.SENSE_KEY & kSENSE_KEY_Mask ),
 								  senseBuffer.ADDITIONAL_SENSE_CODE,
 								  senseBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER ) );
 
@@ -730,12 +907,8 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 							if ( START_STOP_UNIT ( request, 0x00, 0x00, 0x00, 0x01, 0x00 ) == true )
 							{
 								
-								STATUS_LOG ( ( "Sending START_STOP_UNIT.\n" ) );
-								serviceResponse = SendCommand ( request, 0 );
-							}
-							else
-							{
-								PANIC_NOW ( ( "IOSCSIMultimediaCommandsDevice::CheckMediaPresence malformed command" ) );
+								( void ) SendCommand ( request, 0 );
+								
 							}
 							
 							STATUS_LOG ( ( "%s::drive NOT READY\n", getName ( ) ) );
@@ -748,10 +921,12 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 						else if ( ( senseBuffer.ADDITIONAL_SENSE_CODE == 0x3A ) && 
 							 	  ( senseBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER == 0x00 ) )
 						{
+							
 							STATUS_LOG ( ( "No Media.\n" ) );
 							// No media is present, return false
 							driveReady = true;
 							mediaPresent = false;
+							
 						}
 												
 						else
@@ -764,6 +939,7 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
 						}
 										
 					}
+					
 					else if ( ( ( senseBuffer.SENSE_KEY & kSENSE_KEY_Mask ) == kSENSE_KEY_UNIT_ATTENTION ) &&
 								( senseBuffer.ADDITIONAL_SENSE_CODE == 0x28 ) &&
 								( senseBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER == 0x00 ) )
@@ -803,86 +979,103 @@ IOSCSIMultimediaCommandsDevice::CheckMediaPresence ( void )
         
         else
         {
-            // the command failed - perhaps the device was hot unplugged
-            // give other threads some time to run.
+            
+            // The command failed. Let's wait a bit and try again.
             IOSleep ( 200 );
+            
         }
 		
     // check isInactive in case device was hot unplugged during sleep
     // and we are in an infinite loop here
 	} while ( ( driveReady == false ) && ( isInactive ( ) == false ) );
 	
-	bufferDesc->release ( );
 	ReleaseSCSITask ( request );
 	
+	
+ReleaseDescriptor:
+	
+	
+	require_nonzero ( bufferDesc, ErrorExit );
+	bufferDesc->release ( );	
+	bufferDesc = NULL;
+	
+	
+ErrorExit:
+	
+	
 	return mediaPresent;
 	
 }
 
 
-//---------------------------------------------------------------------------
+//
 //  GetCurrentPowerStateOfDrive - 	Gets the drive's current power state
-//---------------------------------------------------------------------------
+//
 
 
 IOReturn
 IOSCSIMultimediaCommandsDevice::GetCurrentPowerStateOfDrive ( UInt32 * powerState )
 {
 	
-	IOMemoryDescriptor *		bufferDesc;
-	SCSITaskIdentifier			request;
-	SCSIServiceResponse 		serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
-	IOReturn					status = kIOReturnError;
 	UInt8						powerStatus[kPowerStatusBufferSize];
+	IOMemoryDescriptor *		bufferDesc		= NULL;
+	SCSITaskIdentifier			request			= NULL;
+	SCSIServiceResponse 		serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
+	IOReturn					status			= kIOReturnError;
 	
 	STATUS_LOG ( ( "%s::%s called\n", getName ( ), __FUNCTION__ ) );
 	
-	if ( fDeviceSupportsPowerConditions )
+	require ( fDeviceSupportsPowerConditions, Exit );
+	
+	bufferDesc = IOMemoryDescriptor::withAddress ( ( void * ) powerStatus,
+													kPowerStatusBufferSize,
+													kIODirectionIn );
+	require_nonzero ( bufferDesc, Exit );
+	
+	request = GetSCSITask ( );
+	require_nonzero ( request, ReleaseDescriptor );
+	
+	if ( GET_EVENT_STATUS_NOTIFICATION ( request,
+										 bufferDesc,
+										 0,
+										 0x02,
+										 kPowerStatusBufferSize,
+										 0 ) == true )
 	{
 		
-		bufferDesc = IOMemoryDescriptor::withAddress ( ( void * ) powerStatus,
-														kPowerStatusBufferSize,
-														kIODirectionIn );
-		
-		request = GetSCSITask ( );
-		
-		if ( GET_EVENT_STATUS_NOTIFICATION ( request,
-											bufferDesc,
-											0,
-											0x02,
-											kPowerStatusBufferSize,
-											0 ) == true )
-		{
-			
-			serviceResponse = SendCommand ( request, 0 );
-			
-		}
-		
-		else
-		{
-			PANIC_NOW ( ( "IOSCSIMultimediaCommandsDevice::GetCurrentPowerStateOfDrive malformed command" ) );
-		}
-		
-		if ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE )
-		{
-			
-			if ( GetTaskStatus ( request ) == kSCSITaskStatus_GOOD )
-			{
-				
-				// According to MMC, the powerStatus byte (6th byte) goes from
-				// high to low, whereas our states go from low to high. Compensate
-				// by subtracting the returned state from the number of power states.
-				*powerState = kMMCNumPowerStates - powerStatus[kPowerStatusByte];
-				status 		= kIOReturnSuccess;
-				
-			}
-			
-		}
-		
-		bufferDesc->release ( );
-		ReleaseSCSITask ( request );
+		serviceResponse = SendCommand ( request, kTenSecondTimeoutInMS );
 		
 	}
 	
+	require ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ), ReleaseTask );
+	require ( ( GetTaskStatus ( request ) == kSCSITaskStatus_GOOD ), ReleaseTask );
+	
+	// According to MMC, the powerStatus byte (6th byte) goes from
+	// high to low, whereas our states go from low to high. Compensate
+	// by subtracting the returned state from the number of power states.
+	*powerState = kMMCNumPowerStates - powerStatus[kPowerStatusByte];
+	status 		= kIOReturnSuccess;
+	
+	
+ReleaseTask:
+	
+	
+	require_nonzero ( request, ReleaseDescriptor );
+	ReleaseSCSITask ( request );
+	request = NULL;
+	
+	
+ReleaseDescriptor:
+	
+	
+	require_nonzero ( bufferDesc, Exit );
+	bufferDesc->release ( );
+	bufferDesc = NULL;
+	
+	
+Exit:
+	
+	
 	return status;
 	
+}
\ No newline at end of file
