@@ -35,7 +35,7 @@
 #include <IOKit/IOService.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSIPrimaryCommands.h>
+#include "SCSIPrimaryCommands.h"
 
 
 //
